#ifndef __ITEXT_H
#define __ITEXT_H 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <iconv.h>

#define EUC_BUFFER	2

#define ChFlag(flag,n)  ((flag & (1 << (n - 1))) ? 1 : 0)   
#define OnFlag(flag,n)  (flag |= (1 << (n - 1)))
#define OfFlag(flag,n)  (flag &= ~(1 << (n - 1)))
#define SetFlag(flag,n) (flag |= (n * 1024))
#define GetFlag(flag,n) (n = (short)(flag >> 10))

#define _EJC    1
#define _CHU    2
#define _SML    3
#define _LST    4
#define _TOP    5
#define _TEN    6
#define _SEN    7
#define _RUB    8
#define _FND    9
#define _KUN   10

typedef enum {
        F_BIP = -1,
        F_NIP,
        F_TXT,
        F_CSV,
        F_ZIP,
        F_HTM
}ftype_t;

typedef enum {
        _U = -1,
        _E,
        _S,
        _J
}fcode_t;

typedef struct ITEXTR{

	unsigned char c1;
	unsigned char c2;
	struct ITEXTR *next, *prev;

}ITEXTR_T;

typedef struct ITEXT{

	unsigned char c1;
	unsigned char c2;
	char *chu_string;
	char off;
	short flag;

	ITEXTR_T *rubi_head,*rubi_tail;
	ITEXTR_T *chu_head,*chu_tail;

	struct ITEXT *next,*prev;

}ITEXT_T;

typedef struct OTHERFILEDATA{

	char *fname;
	struct OTHERFILEDATA *next,*prev;

}IOTHER_T;
typedef struct IBUF{

	short line;
	ITEXT_T *i_head,*i_tail;
	struct IBUF *next,*prev;

}IBUF_T;
typedef struct ICONTENTDATA{

	char *filename;
	char *titlename;

	int total_line;

	IOTHER_T *o_head;
	IBUF_T *b_head,*b_tail;

}ICONTENT_T;

typedef struct SIORIDATA{

        int NPC;

        char book_path[256];
	char book_title[256];
	char book_time[256];

        struct SIORIDATA *next,*prev;

}ISIORI_T;

typedef struct BOOKFILEDATA{

        char fullpath[256];
	char filename[256];
	char title[256];
	char person[256];

        int size,code;

	unsigned long fno;

	char *summary;
	char *bookdir;
	char *dbpath;

	ISIORI_T *siori;

        struct BOOKFILEDATA *next,*prev;

}IBOOK_T;

typedef struct PERSONFILEDATA{

	char *person;
	int no;

	IBOOK_T *head,*tail;

	struct PERSONFILEDATA *next,*prev;

}IPERSON_T;

/* main */
ICONTENT_T *content_link_alloc();
void content_link_free(ICONTENT_T *);
void ibuf_data_free(ICONTENT_T *);
void itext_rubidata_free(ITEXTR_T *);
void itext_data_free(ITEXT_T *);

int _ibuf_add(		IBUF_T **,
			IBUF_T **,
			ITEXT_T *,
			ITEXT_T *,
			int		);

int itextRubiDataInsert(	ITEXTR_T **,
				ITEXTR_T **,
				unsigned char,
				unsigned char	);

int itextDataInsert(	ITEXT_T **,
			ITEXT_T **,
			unsigned char,
			unsigned char,
			int,
			int,
			int		);

int itextGetStrData(	char *,
			ITEXT_T **,
			ITEXT_T **,
			int *,
			int *,
			int *,
			int		);

int _test_01(	FILE *,
		iconv_t,
		ICONTENT_T *,
		int,
		int,
		int			);

int itextGetTextData(ICONTENT_T *,int,int);

/* f1 */
IBOOK_T *book_link_alloc();
void book_link_add(IBOOK_T **,IBOOK_T **,IBOOK_T *);
void person_link_free(IPERSON_T *);
void book_link_free(IBOOK_T **,IBOOK_T **);
int ibookMatchFileData(IBOOK_T **,IBOOK_T **,IBOOK_T *,char *);
int ibookWriteFileData(IBOOK_T *,char *);
int ibookReadFileData(IBOOK_T **,IBOOK_T **,char *);
int ibookGetBookData(IBOOK_T **,IBOOK_T **,char *);
int ibookGetPersonData(IPERSON_T **,IPERSON_T **,IBOOK_T *);

/* f2 */
int checkRubyString(ITEXT_T **,ITEXT_T **,int *,unsigned char *);
int falseSmlString(FILE *,int *,iconv_t);
int falseRubyString(ITEXT_T **,ITEXT_T **);

/* f3 */
int itextCheckEuc(unsigned char *);
char *itextStrStr(char *,char *);
ftype_t itextCheckFileType(char *);
ftype_t itextCheckFileType_ex(char *);
fcode_t itextCheckKanjiCode(FILE *,char *);
iconv_t itextConvOpen(FILE *,char *);
void itextConvClose(iconv_t);
int itextConvKanjiCode(iconv_t,char *,char *,int);
int itextGetTitlePerson(char *,char *,char *);
int itextBufferAlloc(char **,char **,int);
int itextBufferOver(char *,int,int);
int itextBufferCat(char **,char **,char **,int *);

/* f4 */
/* -> xtext.h */

/* f5 */
void other_data_free(IOTHER_T *);
int itextUnZipFile(char *,char **,IOTHER_T **);

/* f6 */
int itextGetZipTitleBuf(char *,char *,int);

/* html */
int _test_02(FILE *,iconv_t,ICONTENT_T *,int,int,int);
int itextGetTitlePerson_htm(char *,char *,char *);

#endif
