#ifndef __XTEXT_
#define __XTEXT_ 


#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/keysym.h>

#if HAVE_LIBXFT

#include <X11/Xft/Xft.h>
#include <X11/extensions/Xrender.h>

#endif

#if HAVE_LIBIMLIB

#include <X11/extensions/shape.h>
#include <Imlib.h>

#endif

#define STEP_MAX 	8
#include <iconv.h>

#ifdef XJP_ICONV
	iconv_t XJP_CONV = NULL;
#else
	extern iconv_t XJP_CONV;
#endif

#define	XJP_OK		"xjp-ok"
#define XJP_CANCEL	"xjp-cancel"
#define XJP_NOOP	"xjp-noop"


enum {
	XJP_NONE = -1,
	XJP_INT,
	XJP_CHA,
	XJP_STR,
	XJP_COL,
	XJP_FONT,
	XJP_ONF
};
enum {
	XJP_LEFT = -1,
	XJP_CENTER,
	XJP_RIGHT
};
enum {
	FTYPE_JPA = 0,
	FTYPE_SMA,
	FTYPE_RUB,
	FTYPE_ENG
};
	
typedef struct OTHERFILEPROG{
	char *name;
	char *exec;
	char *data;
	struct OTHERFILEPROG *next;
	struct OTHERFILEPROG *prev;
}other_prog_t;

typedef struct{
	int main_window_width;
	int main_window_height;

	char main_window_img[128];

	char main_window_color[64];
	char main_font_color[64];
	char main_shadow_color[64];
	char find_font_color[64];
	char rubi_font_color[64];
	char menu_window_color[64];
	char menu_font_color[64];
	char select_window_color[64];
	char select_font_color[64];
	char select_font_background_color[64];
	char pointer_fg_color[64];

	char main_font_j[128];
	char main_font_s[128];
	char main_font_e[128];
	char rubi_font[128];

	char book_dir[128];
	char rast_db[128];
	char ao_url[128];
	char ao_list[128];

	char page_next[8],page_prev[8];
	char conf_page[8],book_mark[8];
	char siori_menu[8];
	char quit[8],menu[8];
	char title_key[8],kinsc_key[8];
	char chuki_key[8],page_key[8];
	char net_key[8],cursor_key[8];
	char other_key[8],find_key[8];
	char find_next[8],find_prev[8];
	char rast_key[8];

	int anti;
	int title_on;
	int ruby_on;
	int page_on;
	int kinsc;
	int cursor;
	int chuki;
	int line_space;
	int row_space;
	int rubi_space;
	int max_line;

	int page_feed0;
	int page_feed1;
	int page_reverse;
	unsigned int page_speed;

	other_prog_t *other_head;
	other_prog_t *other_tail;

}xtext_conf_t;

typedef struct{

	int (*func)(); 
	xtext_conf_t *xconf;

}xtext_func_t;

typedef struct{

	char *book_dir; 
	char *ao_url; 
	char *ao_list;

}xtext_net_t;

typedef struct{
	Window win;
	Pixmap pix0;
	/*Window cwin;*/
	int width,height;

	Cursor pointer1;

	struct {
		Pixmap img;
		int width,height;
		char *position;
	}image;

	GC gc,bgc,sgc;
	GC fgc,rfgc;
	GC rubi_gc,find_gc,black_gc;
	XFontSet fs,fs_s,fs_e,rubi_fs,chu_fs;
	long fg_pixel,bg_pixel;
	long menu_fg_pixel,menu_bg_pixel;
	long rubi_fg_pixel,rubi_bg_pixel;
	long pointer_fg_pixel;

#if HAVE_LIBXFT
	XftFont	*font,*font_s,*font_e,*font_rubi;
	XftDraw	*draw;
	XftColor fg_color,bg_color;
	XftColor rubi_color,find_color;
	XftColor menu_fg_color,menu_bg_color;
#endif

	Visual	*visual;
	Colormap cmap;

	int kinsc;
	int anti;
	int title_on;
	int page_on;
	int cursor;
	int change_font;

	int spf0;
	int spf1;
	int rev;
	unsigned int spl;

}my_window_t;

typedef struct {

	Window parent_win,win;
	int x,y,width,height;
	GC gc,bgc,fgc,rfgc,sgc;
	long fg_pixel,bg_pixel;
	XFontSet fs,fs_s;

#if HAVE_LIBXFT
	XftFont	*font,*font_s;
	XftDraw	*draw;
	XftColor menu_fg_color,menu_bg_color;
#endif

	Visual	*visual;
	Colormap cmap;
	int anti;

}my_subwindow_t;

typedef struct CHARIMAGE{
	unsigned char c1,c2;
	short flag;
	XImage *char_image;
	struct CHARIMAGE *next,*prev;
}image_data_t;
typedef struct PAGEDATA{
	int char_no,page_no;
	struct PAGEDATA *next,*prev;
}page_data_t;
typedef struct {

	char *book,*book_path;
	char *book_title;
	char *book_title_utf8;
	int total_page;

}book_data_t;
typedef struct DRAWDATA{

	int char_header,char_footer;
	int one_line,start_line,end_line;
	int start_low,end_low;

}draw_data_t;

typedef struct CHUDATA{

        char message[256];
	char filepath[10][256];

        struct CHUDATA *next,*prev;

}chu_data_t;

typedef struct {

	int charSize,charSize_s,charSize_e,charSize_r;
	int m_off,s_off,e_off,r_off;
	int xpad,ypad;
	int xpos,ypos;
	draw_data_t *draw;
	book_data_t *book;
	page_data_t *head,*tail;
	page_data_t *find_head,*find_tail;
	image_data_t *im_head,*im_tail;

}char_data_t;

typedef struct SUBWINDOWDATA{

	char *line_str,*re_str;
	void *line_data,*line_data1;
	int (*func)();
	void *func_data0;
	int func_data1;
	int type,line_no,mstr,mdata;
	struct SUBWINDOWDATA *head,*tail;
	struct SUBWINDOWDATA *next,*prev;

}sub_window_t;

typedef struct CTRLWINDOWDATA{
	char *name;
	int xpos0,xpos1,ypos0,ypos1;
	int (*func)();
	void *func_data0;
	struct CTRLWINDOWDATA *next,*prev;
}ctl_window_t;

typedef struct OFFSET_DATA{

        unsigned int c1;
	unsigned int c2;
        char font[1024];
        int anti;
	int x_off,y_off;
        struct OFFSET_DATA *next,*prev;

}offset_data_t;
typedef struct OFFSET{

	offset_data_t *main_off_head, *main_off_tail;
	offset_data_t *sub_off_head, *sub_off_tail;

}offset_t;

typedef struct XJP_FONT_DATA {
        char name[1024];
        struct XJP_FONT_DATA *head,*tail;
        struct XJP_FONT_DATA *next,*prev;
}xtext_font_t;

typedef struct XJP_EFONT_DATA{
        char name[1024];
        struct XJP_EFONT_DATA *next,*prev;
}xtext_efont_t;


/* main */
int config_win(void *,int);
int menu_win(void *,int);

/* f0 */
char *xtext_get_option(int,char **,xtext_conf_t *);
int xtext_create_offsetFile(char *);
/* f1 */
char_data_t * char_data_alloc();
void char_data_free(char_data_t *);
int page_data_insert(page_data_t **,page_data_t **,int,int);
void page_data_free(page_data_t **,page_data_t **);

int xtext_Init_Text(	Display *,
			my_window_t *,
			ICONTENT_T *,
			char_data_t *	);

page_data_t * xtext_Redraw_Text(	Display *,
					my_window_t *,
					ICONTENT_T *,
					char_data_t *,
					int		);
page_data_t * xtext_Init_Book(	Display *,
				my_window_t *,
				ICONTENT_T *,
				char_data_t *,
				offset_t *,
				xtext_conf_t *,
				char *,
				int		);

/* f2 */
int xtext_Draw_Text(	Display *,
			my_window_t *,
			ICONTENT_T *,
			char_data_t *,
			page_data_t *		);

int xtext_Draw_Text_v(	Display *,
			my_window_t *,
			char *[],
			int ,
			int ,
			int			);


/* f3 */
int xtext_prev_kuten(ITEXT_T *,char_data_t *,int);
int xtext_next_kuten(ITEXT_T *,char_data_t *,int);
int xtext_get_low_space(ITEXT_T *,char_data_t *);
int xtext_get_low_pos(ITEXT_T *,char_data_t *,int);

/* f4 */
char *xtextSubWindow(	Display *,
			my_window_t *,
			sub_window_t *,
			sub_window_t *,
			int,
			int		);

int xtextSubWindow_Message(Display *,my_window_t *,char *);

/* f5 */
void xtextClearChuMessage();
int xtextChuMessage(Display *,my_window_t *,char *);
void * xtextChuThreadWidnow(void *);

/* f6 */
char *xtextInputWindow(Display *,my_window_t *,char *,char *,int);

/* f7 */
void SetOffset(char *,int,int);
void GetOffset(char,int *,int *);
void euc2utf8_c(iconv_t,char *,char *,int);
char *euc2utf8(iconv_t,char *,int);
unsigned int xstring2uint(char *);
int checkStepNumber(char *,char [],int);
int Double2Single(char *,char *,int);

/* f8 */
int xtext_get_string_size(	Display *,
				my_window_t *,
				char *,
				char *,
				int *,
				int *,
				int		);

int xtextDrawPageTitleString(	Display *,
				my_window_t *,
				char *,
				char *,
				int ,
				int ,
				int			);

int xtext_Draw_Help(Display *,my_window_t *,xtext_conf_t *);

/* f9 */
void xtext_image_data_free(image_data_t **,image_data_t **);
void xtext_image_data_insert(	Display *,
				my_window_t *,
				char_data_t *,
				ITEXT_T *,
				int		);

int xtext_check_image_data(ITEXT_T *);
XImage * xtext_get_bimage(	Display *,
				my_window_t *,
				char_data_t *	);

int xtextDrawArc(	Display *,
			my_window_t *,
			int,
			int,
			int,
			int,
			int		);

int xtextDrawLine(	Display *,
			my_window_t *,
			int,
			int,
			int,
			int,
			int		);

int xtextDrawFrame(	Display *,
			Window,
			GC,
			int,
			int,
			int		);

/* f10 */
char *xtext_Selector_Window(char *,int);

/* f11 */
int parse_xlfd_No(char *,char *,int);
int xtext_xlfd_create(xtext_font_t **,xtext_font_t **,char *);
int xtext_xlfd_getenco(xtext_efont_t **,xtext_efont_t **,char *);
int xtext_xlfd_free(xtext_font_t *);
int xtext_xlfd_free_e(xtext_efont_t *);
int xtext_font_J2E(char *,char *);

/* f12 */
int _xconf_malloc(char *,int,char *);
char * xtext_conf_data(xtext_conf_t *);
int xtext_conf_change(void *,int);
int xtext_conf_quit(void *,int);
int xtext_conf_save(void *,int);

/* f13 */
int xtext_siori_data_read(ISIORI_T **,ISIORI_T **);
int xtext_siori_data_write(ISIORI_T **,ISIORI_T **);

int xtext_siori_data_mark(	ISIORI_T **,
				ISIORI_T **,
				char *,
				char *,
				int		);

/* f14 */
int xtextCursorType(	Display *,
			my_window_t *,
			char *,
			char *			);

Cursor xtextGetFontCursor(Display *,my_window_t *,char *);

/* f15 */
int xtext_offt_data(offset_data_t **,offset_data_t **,char *);
int xtext_set_offset(IBUF_T *,IBUF_T *,offset_t *);

/* f16 */
void xprint(char *);
void xtext_usage(char *,double,int);
int select_person_data(void *,int);
int select_book_data(void *,int);
int select_serch_data(void *,int);
int select_siori_data(void *,int);
int select_conf_key(void *,int);

/* f17 */
sub_window_t * subwindow_data_alloc();
void * subwindow_data_free(sub_window_t **,sub_window_t **);
void * subwindow_data_add(sub_window_t **,sub_window_t **,sub_window_t *);

void subwindow_data_insert(     sub_window_t **,
                                sub_window_t **,
                                char *,
                                char *,
                                char *,
                                int (*)(),
                                void *,
                                int ,
                                int ,
                                int      		);


int xtextConfInitData2Sub(	sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

int xtextConfFontData2Sub(	sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

int xtextConfWinData2Sub(	sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

int xtextConfKeyData2Sub(	sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

int xtextHelpData2Sub(		sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

int xtextSioriData2Sub(		sub_window_t **,
				sub_window_t **,
				ISIORI_T *		);

int xtextPerson2Sub(		sub_window_t **,
				sub_window_t **,
				IPERSON_T *		);

int xtextBookData2Sub(		sub_window_t **,
				sub_window_t **,
				IBOOK_T	*,
				int 			);

int xtextQuitData2Sub(		sub_window_t **,
				sub_window_t **,
				xtext_conf_t *		);

/* f18 */
#if HAVE_LIBIMLIB

ImlibImage * xtextImlibImage(	Display *,
				ImlibData *,
				ImlibImage **,
				char *			);

int xtextImlibPasteImage(	ImlibData *,
				ImlibImage *,
				Window ,
				int ,
				int			);
#endif


/* i_f4 */
int itextMarkFindData(	IBUF_T *,
			ITEXT_T *,
			page_data_t **,
			page_data_t **	);

int xtextCheckFindData(	ICONTENT_T *,
			char_data_t *,
			char *		);

/* other */
int other_exec(void *,int);
int other_function(void *,int);

int other_win(ICONTENT_T *,xtext_conf_t *);

int xtextOtherData2Sub(	sub_window_t **,
			sub_window_t **,
			IOTHER_T *,
			other_prog_t *		);

int xtextOtherExec2Sub(	sub_window_t **,sub_window_t **,other_prog_t *);

/* etc */
int m_mes(Display *,my_window_t *);

#endif
